@@ -1,5 +1,5 @@
  /* The smooth Class Library
-  * Copyright (C) 1998-2020 Robert Kausch <robert.kausch@gmx.net>
+  * Copyright (C) 1998-2021 Robert Kausch <robert.kausch@gmx.net>
   *
   * This library is free software; you can redistribute it and/or
   * modify it under the terms of "The Artistic License, Version 2.0".
@@ -11,7 +11,10 @@
 #include <gtk/gtk.h>
 
 #include <smooth/backends/xlib/backendxlib.h>
+#include <smooth/files/file.h>
+#include <smooth/foreach.h>
 
+#include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <locale.h>
@@ -22,6 +25,16 @@ using namespace X11;
 #	define GTK_STATE_FLAG_LINK GtkStateFlags(1 << 9)
 #endif
 
+static void toggleTimerInterrupts(bool allowed)
+{
+	sigset_t	 ss;
+
+	sigemptyset(&ss);
+	sigaddset(&ss, SIGALRM);
+
+	pthread_sigmask(allowed ? SIG_UNBLOCK : SIG_BLOCK, &ss, NIL);
+}
+
 static GdkRGBA get_gtk_color(GType type, const gchar *className, GtkStateFlags stateFlags, bool background)
 {
 	/* Create dummy widget of requested type.
@@ -93,10 +106,15 @@ S::Int S::Backends::BackendXLib::Init()
 			return Error();
 		}
 
-		/* Init the GTK.
+		/* Init the GTK with SIGALRM signals disabled to make
+		 * threads created by gtk_init inherit this SIGMASK.
 		 */
+		toggleTimerInterrupts(false);
+
 		gtk_init(NULL, NULL);
 
+		toggleTimerInterrupts(true);
+
 		/* Get default colors.
 		 */
 		UpdateColors();
@@ -181,3 +199,76 @@ XIM S::Backends::BackendXLib::GetIM()
 {
 	return im;
 }
+
+S::String S::Backends::BackendXLib::FindExecutable(const String &executable)
+{
+	/* Search the path for executable.
+	 */
+	String			 path  = getenv("PATH");
+	const Array<String>	&paths = path.Explode(":");
+
+	foreach (const String &path, paths)
+	{
+		/* Check for executable in this path.
+		 */
+		File	 file(String(path).Append("/").Append(executable));
+
+		if (file.Exists()) return file;
+	}
+
+	return NIL;
+}
+
+S::String S::Backends::BackendXLib::QueryGSettings(const String &schema, const String &key)
+{
+	static String	 gsettings = FindExecutable("gsettings");
+
+	if (gsettings == NIL) return NIL;
+
+	/* Execute gsettings to query value.
+	 */
+	FILE	*pstdin = popen(String(gsettings).Append(" get ").Append(schema).Append(" ").Append(key), "r");
+
+	if (pstdin != NIL)
+	{
+		char	 value[256];
+
+		if (fgets(value, 256, pstdin) != NIL)
+		{
+			pclose(pstdin);
+
+			return value;
+		}
+
+		pclose(pstdin);
+	}
+
+	return NIL;
+}
+
+S::String S::Backends::BackendXLib::QueryXfConf(const String &channel, const String &property)
+{
+	static String	 xfconfquery = FindExecutable("xfconf-query");
+
+	if (xfconfquery == NIL) return NIL;
+
+	/* Execute xfconf-query to query value.
+	 */
+	FILE	*pstdin = popen(String(xfconfquery).Append(" -c ").Append(channel).Append(" -p ").Append(property), "r");
+
+	if (pstdin != NIL)
+	{
+		char	 value[256];
+
+		if (fgets(value, 256, pstdin) != NIL)
+		{
+			pclose(pstdin);
+
+			return value;
+		}
+
+		pclose(pstdin);
+	}
+
+	return NIL;
+}
