@@ -1,5 +1,5 @@
  /* The smooth Class Library
-  * Copyright (C) 1998-2020 Robert Kausch <robert.kausch@gmx.net>
+  * Copyright (C) 1998-2021 Robert Kausch <robert.kausch@gmx.net>
   *
   * This library is free software; you can redistribute it and/or
   * modify it under the terms of "The Artistic License, Version 2.0".
@@ -38,35 +38,6 @@ using namespace X11;
 #	define CAIRO_OPERATOR_DIFFERENCE CAIRO_OPERATOR_XOR
 #endif
 
-namespace smooth
-{
-	namespace GUI
-	{
-		static Int PremultiplyAlpha(Bitmap &bitmap)
-		{
-			if (bitmap.GetDepth() != 32) return Success();
-
-			Point	 point;
-			Size	 size = bitmap.GetSize();
-
-			for (point.y = 0; point.y < size.cy; point.y++)
-			{
-				for (point.x = 0; point.x < size.cx; point.x++)
-				{
-					Color	 pixel = bitmap.GetPixel(point);
-
-					if (pixel.GetAlpha() != 255) bitmap.SetPixel(point, Color( pixel.GetRed()   * pixel.GetAlpha() / 255	    |
-												  (pixel.GetGreen() * pixel.GetAlpha() / 255) <<  8 |
-												  (pixel.GetBlue()  * pixel.GetAlpha() / 255) << 16 |
-														      pixel.GetAlpha()	      << 24, Color::RGBA));
-				}
-			}
-
-			return Success();
-		}
-	}
-}
-
 S::GUI::SurfaceBackend *CreateSurfaceCairo(S::Void *iSurface, const S::GUI::Size &maxSize)
 {
 	return new S::GUI::SurfaceCairo(iSurface, maxSize);
@@ -243,13 +214,19 @@ S::Short S::GUI::SurfaceCairo::GetSurfaceDPI() const
 		System::DynamicLoader	 gdk("gdk-3");
 		gint			 (*ex_gdk_screen_get_monitor_scale_factor)(GdkScreen *, gint) = (gint (*)(GdkScreen *, gint)) gdk.GetFunctionAddress("gdk_screen_get_monitor_scale_factor");
 
-		/* Get scale factor.
+		/* Get GDK scale factor.
 		 */
-		Int	 scale = 1.0;
+		Float	 scale = 1.0;
 
 		if (ex_gdk_screen_get_monitor_scale_factor != NIL) scale = ex_gdk_screen_get_monitor_scale_factor(gdk_screen_get_default(), 0);
 		else						   scale = (Int64) Number::FromIntString(getenv("GDK_SCALE"));
 
+		/* Alternatively, query custom DPI setting via XfConf.
+		 */
+		if (scale <= 1.0) scale = Backends::BackendXLib::QueryXfConf("xsettings", "/Xft/DPI").ToInt() / dpi;
+
+		/* Update DPI value if a scale factor has been determined.
+		 */
 		if (scale > 0) dpi *= scale;
 	}
 
@@ -310,33 +287,55 @@ S::Int S::GUI::SurfaceCairo::Line(const Point &iPos1, const Point &iPos2, const
 	Point	 pos1 = rightToLeft.TranslatePoint(iPos1);
 	Point	 pos2 = rightToLeft.TranslatePoint(iPos2);
 
-	/* Adjust to Windows GDI behavior for diagonal lines.
+	/* Draw non-straight lines manually to work around bugs in XWayland.
 	 */
-	if (Math::Abs(pos2.x - pos1.x) == Math::Abs(pos2.y - pos1.y))
+	if (pos1.x != pos2.x && pos1.y != pos2.y)
 	{
-		if (pos1.x > pos2.x) { pos1.x++; pos2.x++; }
-		if (pos1.y > pos2.y) { pos1.y++; pos2.y++; }
-	}
+		Int	 steps	  = Math::Max(Math::Abs(pos2.x - pos1.x), Math::Abs(pos2.y - pos1.y));
+		Float	 xAdvance = steps ? Float(pos2.x - pos1.x) / steps : 0;
+		Float	 yAdvance = steps ? Float(pos2.y - pos1.y) / steps : 0;
 
-	if (!painting)
+		if (!painting)
+		{
+			CreateCairoContext();
+
+			cairo_set_source_rgb(context, color.GetRed() / 255.0, color.GetGreen() / 255.0, color.GetBlue() / 255.0);
+
+			for (Int i = 0; i < steps; i++) cairo_rectangle(context, pos1.x + i * xAdvance, pos1.y + i * yAdvance, 1.0, 1.0);
+		
+			cairo_fill(context);
+
+			DestroyCairoContext();
+		}
+
+		cairo_set_source_rgb(paintContextCairo, color.GetRed() / 255.0, color.GetGreen() / 255.0, color.GetBlue() / 255.0);
+
+		for (Int i = 0; i < steps; i++) cairo_rectangle(paintContextCairo, pos1.x + i * xAdvance, pos1.y + i * yAdvance, 1.0, 1.0);
+		
+		cairo_fill(paintContextCairo);
+	}
+	else
 	{
-		CreateCairoContext();
+		if (!painting)
+		{
+			CreateCairoContext();
 
-		cairo_set_source_rgb(context, color.GetRed() / 255.0, color.GetGreen() / 255.0, color.GetBlue() / 255.0);
-		cairo_set_line_width(context, 1.0);
-		cairo_move_to(context, pos1.x + 0.01, pos1.y + 0.01);
-		cairo_line_to(context, pos2.x + 0.01, pos2.y + 0.01);
-		cairo_stroke(context);
+			cairo_set_source_rgb(context, color.GetRed() / 255.0, color.GetGreen() / 255.0, color.GetBlue() / 255.0);
+			cairo_set_line_width(context, 1.0);
+			cairo_move_to(context, pos1.x + 0.01, pos1.y + 0.01);
+			cairo_line_to(context, pos2.x + 0.01, pos2.y + 0.01);
+			cairo_stroke(context);
 
-		DestroyCairoContext();
+			DestroyCairoContext();
+		}
+
+		cairo_set_source_rgb(paintContextCairo, color.GetRed() / 255.0, color.GetGreen() / 255.0, color.GetBlue() / 255.0);
+		cairo_set_line_width(paintContextCairo, 1.0);
+		cairo_move_to(paintContextCairo, pos1.x + 0.01, pos1.y + 0.01);
+		cairo_line_to(paintContextCairo, pos2.x + 0.01, pos2.y + 0.01);
+		cairo_stroke(paintContextCairo);
 	}
 
-	cairo_set_source_rgb(paintContextCairo, color.GetRed() / 255.0, color.GetGreen() / 255.0, color.GetBlue() / 255.0);
-	cairo_set_line_width(paintContextCairo, 1.0);
-	cairo_move_to(paintContextCairo, pos1.x + 0.01, pos1.y + 0.01);
-	cairo_line_to(paintContextCairo, pos2.x + 0.01, pos2.y + 0.01);
-	cairo_stroke(paintContextCairo);
-
 	return Success();
 }
 
